GtkWidget* create_window1 (void);
GtkWidget* create_menu1 (void);
GtkWidget* create_dialog1 (void);
GtkWidget* create_window2 (void);
GtkWidget* create_dialog2 (void);
GtkWidget* create_dialog3 (void);
GtkWidget* create_dialog4 (void);
GtkWidget* create_window3 (void);
GtkWidget* create_dialog6 (void);
GtkWidget* create_window5 (void);
GtkWidget* create_window6 (void);
GtkWidget* create_window8 (void);
GtkWidget* create_window10 (void);
GtkWidget* create_dialog7 (void);
GtkWidget* create_window12 (void);
GtkWidget* create_dialog8 (void);
GtkWidget* create_win13_biggeo (void);
GtkWidget* create_dialog_geocode (void);
GtkWidget* create_filechooserdialog1 (void);
GtkWidget* create_dialog_image_data (void);
GtkWidget* create_dialog_geocode_result (void);
GtkWidget* create_dialog9 (void);
GtkWidget* create_dialog10 (void);
